/* 	DFT.H	/ Borlan C/C++ 3.1 /

	Sadrzi rutine za racunanje DFT/FFT digitalnog signala.

	Pavlovic Dejan, 1998.
*/

#ifndef _DFT_H_
#define _DFT_H_

typedef struct {
   float re, im;
} cplx;

enum window_type {WIN_NONE, WIN_BARTLETT, WIN_SINE, WIN_HANU,
		  WIN_HAMMING, WIN_BLACKMAN, WIN_KAISER, WIN_LOAD};

void set_float_array(float far a[], int N, float value);
/* U niz upisuje vrednost value. */

void scale_float_array(float far a[], int N, float scale);
/* Skalira niz duzine N */

double I0(double x);
/* Baselova funkcija nultog reda. */

void make_window(float far w[], int N_win, int N, int scale,
		 enum window_type type,
		 double kaiser_beta,
		 float far w_load[], int N_w_load);
/* Kreira odgovarajucu prozorsku funkciju, duzine N_win i niz duzine N */
/* (N je duzina sempla). Ako je scale aktivno, vrsi se skaliranje tako */
/* da je sum(w[0:N_win-1]) = N*1 = N. Prom. window_type odrdjuje tip */
/* prozorske f-je. Ako je kaizerova onda se koristi param. kaiser_beta */
/* Ako je tipa LOAD onda se u w[0:N_win-1] upisuje w_load[0:N_w_load-1] */
/* Pri semu se vrsi linearna interpolacija ako je N_win <> N_w_load */

void make_binindex(int far binindex[], int N, int dft_type);
/* Kreira niz binindex[] koji sadrzi informaciju kako se prerspodeljuju */
/* semplovi u vremenskom domenu u zavsnosti od tipa DFT. */

void make_cossin_table(float far cosfi[], float far sinfi[], int N);
/* kreira tabelu kosinusa i sinusa potrebnih za izracunavanje DFT/FFT. */

void sample2bincomplex(float sample[], int far binindex[], float far w[],
		       cplx far x[], int N);
/* Konvertuje sempl u komleksne brojeve. Koristi prozorske f-je */
/* i vrsi odgovarajuce (binarno) premestanje u vremenskom domenu. */

void realsample2bincomplex(float sample[], int far binindex[], float far w[],
			   cplx far x[], int N);
/* Konvertuje sempl duzine N u komleksnu sekvencu duzine N/2 */
/* radi brzeg izracunavanja DFT/FFT. Posle DFT/FFT treba izvrsiti */
/* korekciju. Koristi prozorske f-je i vrsi odgovarajuce */
/* (binarno) premestanje u vremenskom domenu (premestanje za FFT). */

void correct_complex(cplx far x[], int N_new, cplx far *xp,
		     float far *pcos, float far *psin);
/* Ulaz je x = DFT(s1[] + s2[]*j), gde su s1[] i s2[] parni i neparni */
/* clanovi realnog niza. x trba modifikovati tako da se dobije spektar */
/* orginalne sekvence. xp[] je pomocna memorija duzine 2*size(x). */
/* pcos i psin su pokaz. na tabele cos/sin koje se koriste za izrac. FFT */

void DFT(cplx far x[], cplx far f[], int N);
/* Orginalna funkcija za racunanje DFT, bez ikakvih optimizacija. */

void FFT1(cplx far x[], int N);
/* Klasicni FFT radiks-2 algoritam sa preuredjenjem u vremenu (DIT). */

void FFT2(cplx far x[], int N);
/* Klasicni FFT radiks-2 algoritam sa preuredjenjem u vremenu (DIT). */
/* sinusi/kosinusi racunaju se rekurzivno: brze ali unosi malu gresku. */

void FFT3(cplx far x[], int N, float far cosfi[], float far sinfi[]);
/* Klasicni FFT radiks-2 algoritam sa preuredjenjem u vremenu (DIT). */
/* Kosinusi i sinusi uzimaju se iz tabele. */

void FFT4(cplx far x[], int N, float far cosfi[], float far sinfi[]);
/* Klasicni FFT radiks-2 algoritam sa preuredjenjem u vremenu (DIT). */
/* Kosinusi i sinusi uzimaju se iz tabele. Koristi pointere. */

double get_f0(cplx far f[], int size, double sync_level, int h_end);
/* Braca digitalnu frekvenciju osnovnog harmonika. */
/* Prvo pronadje maksimalnu frekv. komponentu koja je veca od sync_level */
/* To je PRIBLIZNO osnovna frekvencija signala. */
/* Zatim ispita maksimume u okolinama frekvencijskih komponenti koje su */
/* multipli osnovne frekvencije. Na kraju se izracuna prosek odstojanja */
/* frekv. maksimuma (harmonici) i to je osnovna frekvencija signala */
/* Prilikom racunanja proseka uzima se tezinska srednja vrednost, tj. */
/* frekvencijska komponenta sa vecom amplitudom vise utice na prosek, dok */
/* oni maksimumi koji su manji od sync_level uopste ne uticu na prosek. */
/* Ako je h_end >= 1 razmatra se maksimalno h_end harmonika, u protivnom */
/* svi postojeci (h_end <= 0). */
/* Ovo vazi akoje sync_level > 0.0. Ako je = 0.0 onda se vraca frekvencija */
/* onog harmonika koji ima najvecu amplitudu */

cplx get_fourier(cplx far x[], int N, double omega);
/* Na osnovu DFT sekvence x[] daje furijeovu transformaciju dig. signala. */

#endif
